@@ -1,94 +1,94 @@
-# Written by HJK Group
-# Dpt of Chemical Engineering, MIT
+## @file inparse.py
+#  Processes inputs
+#
+#  Written by Tim Ioannidisfor HJK Group
+#
+#  Dpt of Chemical Engineering, MIT
 
-# Note to developers: when adding new keywords, update both parseinputfile as well as parseinputs_CAT.
-
-##############################################################
-########## This script processes the input file  #############
-##############################################################
-
-# import std modules
 import glob, os, re, argparse, sys, ast
 from molSimplify.Scripts.io import *
 from molSimplify.Classes.globalvars import *
 from pkg_resources import resource_filename, Requirement
 
-######################################################
-########## check core/ligands specified  #############
-######################################################
-### checks input for correctness and uses defaults otherwise ### -> How do we do this for the GUI? Also make sure this doesn't break for other features e.g. slab builder
+## Checks input for correctness and uses defaults otherwise
+#  @param args Namespace of arguments
 def checkinput(args):
-    globs = globalvars()
-    emsg = False
-    # check core
-    if not args.core:
-        print 'WARNING: No core specified. Defaulting to Fe. Available cores are: '+getcores()
-        args.core = ['fe']
-    # check oxidation state
-    if not args.oxstate:
-        try:
-            print 'WARNING: No oxidation state specified. Defaulting to '+globs.defaultoxstate[args.core[0].lower()]
-            args.oxstate = globs.defaultoxstate[args.core[0].lower()]
-        except:
-            print 'WARNING: No oxidation state specified. Defaulting to 2'
-            args.oxstate = '2'
-    # check spin state (doesn't work for custom cores)
-    if not args.spin:
-        if args.oxstate in romans.keys():
-            oxstatenum = romans[args.oxstate]
-        else:
-            oxstatenum = args.oxstate
-        if args.core[0].lower() in mtlsdlist:
-            if mtlsdlist[args.core[0].lower()]-max(0,int(oxstatenum)-2) in defaultspins:
-                defaultspinstate = defaultspins[mtlsdlist[args.core[0].lower()]-max(0,int(oxstatenum)-2)]
-                print 'WARNING: No spin multiplicity specified. Defaulting to '+defaultspinstate
-                print 'Please check this against our ANN output (where available)'
-            else:
-                print 'WARNING: Oxidation state seems to be invalid. Please check. Defaulting to singlet anyway.' 
-                defaultspinstate = '1'
-        else:
-            defaultspinstate = '1'
-            print 'WARNING: No spin multiplicity specified. Defaulting to singlet (1)'
-        args.spin = defaultspinstate
-    # check ligands
-    if not args.lig and not args.rgen:
-        if args.gui:
-            from Classes.mWidgets import mQDialogWarn
-            qqb = mQDialogWarn('Warning','You specified no ligands.')
-            qqb.setParent(args.gui.wmain)
-        else:
-            print 'WARNING: No ligands specified. Defaulting to water.'
-        args.lig = ['water']
-    # check coordination number and geometry
-    if not args.coord and not args.geometry:
-        if not args.gui:
-            print 'WARNING: No geometry and coordination number specified. Defaulting to octahedral (6).'
-            args.coord = 6
-            args.geometry = 'oct'
-    coords,geomnames,geomshorts,geomgroups = getgeoms()
-    if args.coord and (not args.geometry or (args.geometry not in geomnames and args.geometry not in geomshorts)):
-        print 'WARNING: No or unknown coordination geometry specified. Defaulting to '+globs.defaultgeometry[int(args.coord)][1]
-        args.geometry = globs.defaultgeometry[int(args.coord)][0]
-    if args.geometry and not args.coord:
-        if args.geometry not in geomnames and args.geometry not in geomshorts:
-            print 'You have specified an invalid geometry. Available geometries are:'
-            printgeoms()
-            print 'Defaulting to octahedral (6)'
-            args.geometry = 'oct'
-            args.coord = 6
-        else:
-            try:
-                args.coord = coords[geomnames.index(args.geometry)]
-            except:
-                args.coord = coords[geomshorts.index(args.geometry)]
-            print 'WARNING: No coordination number specified. Defaulting to '+str(args.coord)
-    # check number of ligands
-    if args.coord and not args.ligocc:
-        print('WARNING: No ligand numbers specified. Defaulting to '+str(args.coord)+' of the first ligand and 0 of all others.')
-        args.ligocc = [args.coord]
-        for lig in args.lig[1:]:
-            args.ligocc.append(0)
+	globs = globalvars()
+	emsg = False
+	# check core
+	if not args.core:
+	    print 'WARNING: No core specified. Defaulting to Fe. Available cores are: '+getcores()
+	    args.core = ['fe']
+	if args.core[0][0].upper()+args.core[0][1:].lower() in elementsbynum:    
+	    # convert to titlecase
+	    args.core[0] = args.core[0][0].upper()+args.core[0][1:].lower()
+	    # check oxidation state
+	    if not args.oxstate:
+	        try:
+	            print 'WARNING: No oxidation state specified. Defaulting to '+globs.defaultoxstate[args.core[0].lower()]
+	            args.oxstate = globs.defaultoxstate[args.core[0].lower()]
+	        except:
+	            print 'WARNING: No oxidation state specified. Defaulting to 2'
+	            args.oxstate = '2'
+	    # check spin state (doesn't work for custom cores)
+	    if not args.spin:
+	        if args.oxstate in romans.keys():
+	            oxstatenum = romans[args.oxstate]
+	        else:
+	            oxstatenum = args.oxstate
+	        if args.core[0].lower() in mtlsdlist:
+	            if mtlsdlist[args.core[0].lower()]-max(0,int(oxstatenum)-2) in defaultspins:
+	                defaultspinstate = defaultspins[mtlsdlist[args.core[0].lower()]-max(0,int(oxstatenum)-2)]
+	                print 'WARNING: No spin multiplicity specified. Defaulting to '+defaultspinstate
+	                print 'Please check this against our ANN output (where available)'
+	            else:
+	                print 'WARNING: Oxidation state seems to be invalid. Please check. Defaulting to singlet anyway.' 
+	                defaultspinstate = '1'
+	        else:
+	            defaultspinstate = '1'
+	            print 'WARNING: No spin multiplicity specified. Defaulting to singlet (1)'
+	        args.spin = defaultspinstate
+	    # check ligands
+	    if not args.lig and not args.rgen:
+	        if args.gui:
+	            from Classes.mWidgets import mQDialogWarn
+	            qqb = mQDialogWarn('Warning','You specified no ligands.')
+	            qqb.setParent(args.gui.wmain)
+	        else:
+	            print 'WARNING: No ligands specified. Defaulting to water.'
+	        args.lig = ['water']
+	    # check coordination number and geometry
+	    if not args.coord and not args.geometry:
+	        if not args.gui:
+	            print 'WARNING: No geometry and coordination number specified. Defaulting to octahedral (6).'
+	            args.coord = 6
+	            args.geometry = 'oct'
+	    coords,geomnames,geomshorts,geomgroups = getgeoms()
+	    if args.coord and (not args.geometry or (args.geometry not in geomnames and args.geometry not in geomshorts)):
+	        print 'WARNING: No or unknown coordination geometry specified. Defaulting to '+globs.defaultgeometry[int(args.coord)][1]
+	        args.geometry = globs.defaultgeometry[int(args.coord)][0]
+	    if args.geometry and not args.coord:
+	        if args.geometry not in geomnames and args.geometry not in geomshorts:
+	            print 'You have specified an invalid geometry. Available geometries are:'
+	            printgeoms()
+	            print 'Defaulting to octahedral (6)'
+	            args.geometry = 'oct'
+	            args.coord = 6
+	        else:
+	            try:
+	                args.coord = coords[geomnames.index(args.geometry)]
+	            except:
+	                args.coord = coords[geomshorts.index(args.geometry)]
+	            print 'WARNING: No coordination number specified. Defaulting to '+str(args.coord)
+	    # check number of ligands
+	    if args.coord and not args.ligocc:
+	        print('WARNING: No ligand numbers specified. Defaulting to '+str(args.coord)+' of the first ligand and 0 of all others.')
+	        args.ligocc = [args.coord]
+	        for lig in args.lig[1:]:
+	            args.ligocc.append(0)
 
+## Checks TS builder input for correctness and uses defaults otherwise
+#  @param args Namespace of arguments
 def checkinput_ts(args):
     globs = globalvars()
     emsg = False
@@ -110,9 +110,9 @@ def checkinput_ts(args):
         print 'WARNING: No spin multiplicity specified. Defaulting to singlet (1)'
         args.spin = '1'
         
-###########################################
-########## check true or false  ###########
-###########################################
+## Check true or false
+#  @param arg String to be checked    
+#  @return bool
 def checkTrue(arg):
 
     if 'auto' in arg.lower():
@@ -122,9 +122,9 @@ def checkTrue(arg):
     else:
         return False
 
-########################################
-########## check for number  ###########
-########################################
+## Check if variable is a number
+#  @param s variable to be checked
+#  @return bool
 def is_number(s):
     try:
         float(s)
@@ -139,10 +139,8 @@ def is_number(s):
         pass
     return False
 
-###########################################
-########## consolidate lists  #############
-###########################################
-### consolidate arguments ###
+## Consolidate arguments into lists
+#  @param args Namespace of arguments
 def cleaninput(args):
     globs = globalvars()
     # check ligands
@@ -242,7 +240,6 @@ def cleaninput(args):
                 args.keepHs[i]=checkTrue(s)
     # parse FF settings:
     # if no FF opt is requested, turn off
-    print args.ffoption
     if args.ffoption[0].lower() == ('n' or 'no'):
         args.ff = False
     # if FF opt is desired, parse FF choice
@@ -265,11 +262,9 @@ def cleaninput(args):
                     if op[0].lower()=='a':
                         args.ffoption += 'a'
 
-
-###################################################
-##########  parse command line input  #############
-###################################################
-### parses inputfile ###
+## Generates input file from command line input
+#  @param args Namespace of arguments
+#  @return CLIinput.inp (file name)
 def parseCLI(args):
     cliargs = ' '.join(args)
     s = filter(None,cliargs.split('-'))
@@ -281,10 +276,8 @@ def parseCLI(args):
     f.close()
     return fname
 
-###########################################
-##########  parse input file  #############
-###########################################
-### parses inputfile ###
+## Parses input file
+#  @param args Namespace of arguments
 def parseinputfile(args):
     #### arguments that don't match with  inparse name and
     ### are not automatically initialized go here:
@@ -405,7 +398,7 @@ def parseinputfile(args):
                 args.ff = l[1].lower()
             if (l[0]=='-ffoption' and len(l[1:]) > 0):
                 args.ffoption = l[1:]
-                print('setting ffoption ' + str(args.ffoption))
+                #print('setting ffoption ' + str(args.ffoption))
             if (l[0]=='-place' and len(l[1:]) > 0):
                 args.place = l[1]
             if (l[0]=='-sminame' and len(l[1:]) > 0):
@@ -434,7 +427,12 @@ def parseinputfile(args):
                     args.smicat.append(lloc)    
                 
                 print('final smicat set to ' + str(args.smicat))
-                
+            if (l[0]=='-nconfs' and len(l[1:]) > 0):
+                args.nconfs = l[1]    
+            #if (l[0]=='-maxconfs' and len(l[1:]) > 0):
+                #args.maxconfs = l[1]
+            if (l[0]=='-scoreconfs'):
+                args.scoreconfs = True                                
             if '-pangles' in line:
                 args.pangles = []
                 l = filter(None,line.split('pangles',1)[1])
@@ -691,7 +689,9 @@ def parseinputfile(args):
             if (l[0]=='-substreact'):
                 args.substreact = l[1:]
             if (l[0]=='-drawmode'):
-                args.drawmode = True                                      
+                args.drawmode = True 
+            if (l[0]=='-substplaceff'):
+                args.substplaceff = True                                                      
             # parse place on slab options
             if (l[0]=='-place_on_slab'): #0
                 args.place_on_slab = True
@@ -749,11 +749,9 @@ def parseinputfile(args):
 	    if (l[0] == '-max_descriptors'):
 		args.max_descriptors = [str(i) for i in l[1:]]
 		
-
-#############################################################
-########## mainly for help and listing options  #############
-#############################################################
-### parses commandline arguments and prints help information ###
+## Parses command line arguments and prints help information
+#  @param parser Parser object
+#  @return Namespace of arguments
 def parseall(parser):
     globs = globalvars()
     parser.add_argument("-i", help="input file")
@@ -778,8 +776,11 @@ def parseall(parser):
     parseinputs_naming(parser,args)
     return args
 
+## Parses basic input options and prints help
+#  @param *p Parser pointer
 def parseinputs_basic(*p):
     parser = p[0]
+    parser.add_argument("-core", help="core structure with currently available: "+getcores())
     parser.add_argument("-oxstate", help="oxidation state of the metal") # specified in cleaninput
     parser.add_argument("-coord", help="coordination such as 4,5,6",action="store_true")
     parser.add_argument("-geometry", help="geometry",action="store_true")
@@ -794,12 +795,17 @@ def parseinputs_basic(*p):
         args = p[1]
         parser.parse_args(namespace=args)
         return 0
-
+        
+## Parses advanced input options and prints help
+#  @param *p Parser pointer
 def parseinputs_advanced(*p):
     parser = p[0]
     # advanced structure generation options
     parser.add_argument("-rundir", help="directory for jobs, default ~/Runs",action="store_true")
     parser.add_argument("-smicat", help="connecting atoms corresponding to smiles. Indexing starts at 1 which is the default value as well. Use [] for multiple SMILES ligands, e.g. [1],[2]",action="store_true")
+    parser.add_argument("-nconfs", help="Number of conformers to generate for multidentate smiles ligands. Default 1.", default='1')
+    parser.add_argument("-scoreconfs", help="Attempt to filter out identical conformers and rank them by rmsd to the desired template, default false", default=False)
+    #parser.add_argument("-maxconfs", help="Stop generation after maxconfs unique conformers or nconfs conformers have been generated, whichever comes first, default infinite", default=10000)
     parser.add_argument("-charge", help="Net complex charge. Recommended NOT to specify, by default this is calculated from the metal oxidation state and ligand charges.")
     parser.add_argument("-calccharge", help="Automatically calculate net complex charge. By default this is ON.", default=True)
     parser.add_argument("-ff", help="select force field for FF optimization. Available: (default) MMFF94, UFF, GAFF, Ghemical",default='mmff94')
@@ -808,7 +814,7 @@ def parseinputs_advanced(*p):
     parser.add_argument("-ligloc", help="force location of ligands in the structure generation (default False)",default=False)
     parser.add_argument("-decoration_index", help="list of indicies on each ligand to decorate",action="store_true") # decoration indexes, one list per ligand
     parser.add_argument("-decoration", help="list of SMILES for each decoratation",action="store_true") # decoration, one list ligand  
-    parser.add_argument("-ligalign", help="smart alignment of ligands in the structure generation (default True)",default=False)
+    parser.add_argument("-ligalign", help="smart alignment of ligands in the structure generation (default False)",default=False)
     parser.add_argument("-MLbonds", help="custom M-L bond length (Ang) for corresponding ligand",action="store_true")
     parser.add_argument("-distort", help="randomly distort backbone. Ranges from 0 (no distortion) to 100. e.g. 20",default='0')
     parser.add_argument("-langles", help="custom angles (polar theta, azimuthal phi) for corresponding ligand in degrees separated by '/' e.g. 20/30,10/20",action="store_true")
@@ -821,6 +827,8 @@ def parseinputs_advanced(*p):
         parser.parse_args(namespace=args)
         return 0
 
+## Parses slab builder options and prints help
+#  @param *p Parser pointer
 def parseinputs_slabgen(*p):
     parser = p[0]
     # slab builder input: controli
@@ -864,6 +872,8 @@ def parseinputs_slabgen(*p):
         parser.parse_args(namespace=args)
         return 0
 
+## Parses automated correlation analysis options and prints help
+#  @param *p Parser pointer
 def parseinputs_autocorr(*p):
     parser = p[0]
     parser.add_argument('-correlate', help = "path to file for analysis, should contain name,value,folder where name.xyz geo is located on each line") #0
@@ -878,6 +888,8 @@ def parseinputs_autocorr(*p):
         parser.parse_args(namespace=args)
     return 0
 
+## Parses chain builder options and prints help
+#  @param *p Parser pointer
 def parseinputs_chainb(*p):
     parser = p[0]
     parser.add_argument('-chain', help = "SMILES string of monomer", action = "store_true") #0
@@ -890,6 +902,8 @@ def parseinputs_chainb(*p):
         parser.parse_args(namespace=args)
     return 0
 
+## Parses database search options and prints help
+#  @param *p Parser pointer
 def parseinputs_db(*p):
     parser = p[0]
     parser.add_argument("-dbsearch", help="flag enabling db search",action="store_true") 
@@ -923,6 +937,8 @@ def parseinputs_db(*p):
         parser.parse_args(namespace=args)
     return 0
 
+## Parses input file generation options and prints help
+#  @param *p Parser pointer
 def parseinputs_inputgen(*p):
     parser = p[0]
     parser.add_argument("-qccode", help="Code to use, only terachem (default), GAMESS and qchem are currently supported", default='terachem')
@@ -965,6 +981,8 @@ def parseinputs_inputgen(*p):
         parser.parse_args(namespace=args)
     return 0
 
+## Parses postprocessing options and prints help
+#  @param *p Parser pointer
 def parseinputs_postproc(*p):
     parser = p[0]
     parser.add_argument("-postp", help="post process results",action="store_true")
@@ -987,6 +1005,8 @@ def parseinputs_postproc(*p):
         parser.parse_args(namespace=args)
     return 0
 
+## Parses random generation options and prints help
+#  @param *p Parser pointer
 def parseinputs_random(*p):
     parser = p[0]
     parser.add_argument("-liggrp","--liggrp", help="ligand group for random generation ",action="store_true")
@@ -1002,6 +1022,8 @@ def parseinputs_random(*p):
         parser.parse_args(namespace=args)
     return 0 
 
+## Parses binding species placement options and prints help
+#  @param *p Parser pointer
 def parseinputs_binding(*p):
     parser = p[0]
     parser.add_argument("-bind", help="binding species, currently available: "+getbinds(),action="store_true")
@@ -1023,6 +1045,8 @@ def parseinputs_binding(*p):
         parser.parse_args(namespace=args)
     return 0 
 
+## Parses transition state building options and prints help
+#  @param *p Parser pointer
 def parseinputs_tsgen(*p):
     parser = p[0]
     parser.add_argument("-tsgen", help="flag for enabling TS generation mode",action="store_true")
@@ -1030,6 +1054,7 @@ def parseinputs_tsgen(*p):
     parser.add_argument("-compreact", help="index of reacting atom in core")
     parser.add_argument("-substreact", help="index of reacting atom(s) in substrate")
     parser.add_argument("-drawmode", help="flag for enabling draw mode (NOTE: this cancels TS generation)",action="store_true")
+    parser.add_argument("-substplaceff", help="full FF opt at each possible connecting point (default False - uses empirically estimated sterics)",default=False)
     if len(p) == 1: # only one input, printing help only
         args = parser.parse_args()
         return args
@@ -1038,9 +1063,10 @@ def parseinputs_tsgen(*p):
         parser.parse_args(namespace=args)
     return 0 
 
+## Parses ligand replacement options and prints help
+#  @param *p Parser pointer
 def parseinputs_customcore(*p):
     parser = p[0]
-    parser.add_argument("-core", help="core structure with currently available: "+getcores())
     parser.add_argument("-replig", help="flag for replacing ligand at specified connection point",action="store_true")
     parser.add_argument("-ccatoms", help="core connection atoms indices, indexing starting from 1")
     if len(p) == 1: # only one input, printing help only
@@ -1051,6 +1077,8 @@ def parseinputs_customcore(*p):
         parser.parse_args(namespace=args)
     return 0 
 
+## Parses file naming options and prints help
+#  @param *p Parser pointer
 def parseinputs_naming(*p):
     parser = p[0]
     parser.add_argument("-name", help="custom name for complex",action="store_true")
