@@ -1,20 +1,17 @@
-# Written by Tim Ioannidis for HJK Group
-# modified by JP Janet
-# Dpt of Chemical Engineering, MIT
+## @file io.py
+#  Input/output functions
+#  
+#  Written by Tim Ioannidis for HJK Group
+#  
+#  Dpt of Chemical Engineering, MIT
 
-##########################################################
-########## This script handles input/output  #############
-##########################################################
-
-# import std modules
 import glob, os,shutil, re, argparse, sys, random, openbabel
 from molSimplify.Classes.mol3D import *
 from molSimplify.Classes.globalvars import *
+from molSimplify.Classes.mol3D import mol3D
 from pkg_resources import resource_filename, Requirement
 
-##############################################
-### function to print available geometries ###
-##############################################
+## Print available geometries
 def printgeoms():
     globs = globalvars()
     if globs.custom_path:
@@ -41,12 +38,9 @@ def printgeoms():
         print "Coordination: %s, geometry: %s,\t short name: %s " %(coords[i],g,geomshorts[i])
     print ''
     
-##############################################
-### function to get available geometries ###
-##############################################
+## Get available geometries
 def getgeoms():
     globs = globalvars()
-#   f = open(globs.installdir+'/Data/coordinations.dict','r')
     if globs.custom_path:
         f = globs.custom_path + "/Data/coordinations.dict"
     else:
@@ -70,12 +64,12 @@ def getgeoms():
     for i in range(1,len(coords)):
         if coords[i-1]!=coords[i]:
             count += 1
-        geomgroups[count].append(geomshorts[i])
+        geomgroups[count].append(geomshorts[i])   
     return coords,geomnames,geomshorts,geomgroups
 
-###################################
-### function to read dictionary ###
-###################################
+## Read data into dictionary
+#  @param fname Filename containing dictionary data
+#  @return Dictionary
 def readdict(fname):
     d = dict()
     f = open(fname,'r')
@@ -97,9 +91,8 @@ def readdict(fname):
             d[key] = vv
     return d 
 
-##############################
-### get ligands dictionary ###
-##############################
+## Get ligands in dictionary
+#  @return List of ligands in dictionary
 def getligs():
     licores = getlicores()
     a=[]
@@ -108,15 +101,12 @@ def getligs():
     a = sorted(a)
     a = ' '.join(a)
     return a
-##############################
-### get ligands cores      ###
-##############################
+    
+## Get ligands cores
+#
+#  This form of the function is used extensively in the GUI so it got it's own call. This is basically the same as getligs() but returns the full dictionary
+#  @return Ligands dictionary
 def getlicores():
-    ## this form of the functionn
-    ## is used extensively in the GUI
-    ## so it got it's own call. This
-    ## is basically the same as getligs but
-    ## returns the full dictionary
     globs = globalvars()
     if globs.custom_path: # test if a custom path is used:
          licores = str(globs.custom_path).rstrip('/') + "/Ligands/ligands.dict"
@@ -124,9 +114,9 @@ def getlicores():
         licores = resource_filename(Requirement.parse("molSimplify"),"molSimplify/Ligands/ligands.dict")
     licores = readdict(licores)
     return licores
-##############################
-### get simple ligands dict###
-##############################
+
+## Get simple ligands in dictionary
+#  @return List of ligands in simple ligands dictionary
 def getsimpleligs():
     slicores = getslicores()
     for key in slicores:
@@ -134,9 +124,11 @@ def getsimpleligs():
     a = sorted(a)
     a = ' '.join(a)
     return a
-##############################
-### get simple ligands core###
-##############################
+    
+## Get simple ligands cores
+#
+#  This form of the function is used extensively in the GUI so it got it's own call. This is basically the same as getsimpleligs() but returns the full dictionary
+#  @return Simple ligands dictionary
 def getslicores():
     globs = globalvars()
     if globs.custom_path: # test if a custom path is used:
@@ -145,9 +137,10 @@ def getslicores():
         slicores = resource_filename(Requirement.parse("molSimplify"),"molSimplify/Ligands/simple_ligands.dict")
     slicores = readdict(slicores)
     return slicores
-#########################
-### get ligand groups ###
-#########################
+    
+## Get ligand groups
+#  @param licores Ligand dictionary
+#  @return Ligand groups
 def getligroups(licores):
     groups = []
     for entry in licores:
@@ -156,9 +149,9 @@ def getligroups(licores):
     a = ' '.join(groups)
     return a
     
-###################################
-### put [] on metals for SMILES ###
-###################################
+## Enclose metal elements in SMILES string with square brackets
+#  @param smi SMILES string
+#  @return Processed SMILES string
 def checkTMsmiles(smi):
     g = globalvars()
     for m in g.metals():
@@ -166,9 +159,8 @@ def checkTMsmiles(smi):
             smi = smi.replace(m,'['+m+']')
     return smi
 
-##############################
-### get bind dictionary ###
-##############################
+## Get binding species in dictionary
+#  @return List of binding species in dictionary
 def getbinds():
     bindcores = getbcores() 
     a=[]
@@ -177,15 +169,12 @@ def getbinds():
     a = sorted(a)
     a = ' '.join(a)
     return a
-#############################
-### get bind cores      ###
-##############################
+    
+## Get binding species cores
+#
+#  This form of the function is used extensively in the GUI so it got it's own call. This is basically the same as getbinds() but returns the full dictionary
+#  @return Binding species dictionary
 def getbcores():
-    ## this form of the functionn
-    ## is used extensively in the GUI
-    ## so it got it's own call. This
-    ## is basically the same as getbinds() but
-    ## returns the full dictionary
     globs = globalvars()
     if globs.custom_path: # test if a custom path is used:
         bcores = str(globs.custom_path).rstrip('/') + "/Bind/bind.dict"
@@ -193,9 +182,9 @@ def getbcores():
         bcores = resource_filename(Requirement.parse("molSimplify"),"molSimplify/Bind/bind.dict")
     bcores = readdict(bcores)
     return bcores
-############################
-### get cores dictionary ###
-############################
+    
+## Get cores in dictionary
+#  @return List of cores in dictionary
 def getcores():
     mcores = getmcores()
     a=[]
@@ -204,15 +193,12 @@ def getcores():
     a = sorted(a)
     a = ' '.join(a)
     return a
-#############################
-### get mcores            ###
-##############################
+    
+## Get core cores
+#
+#  This form of the function is used extensively in the GUI so it got it's own call. This is basically the same as getcores() but returns the full dictionary
+#  @return Cores dictionary
 def getmcores():
-    ## this form of the functionn
-    ## is used extensively in the GUI
-    ## so it got it's own call. This
-    ## is basically the same as getcores() but
-    ## returns the full dictionary
     globs = globalvars()
     if globs.custom_path: # test if a custom path is used:
          mcores = str(globs.custom_path).rstrip('/') + "/Cores/cores.dict"
@@ -220,9 +206,9 @@ def getmcores():
         mcores = resource_filename(Requirement.parse("molSimplify"),"molSimplify/Cores/cores.dict")
     mcores = readdict(mcores)
     return mcores
-############################
-### get substrates dictionary ###
-############################
+    
+## Get substrates in dictionary
+#  @return List of substrates in dictionary
 def getsubstrates():
     subcores = getsubcores()
     a=[]
@@ -231,15 +217,12 @@ def getsubstrates():
     a = sorted(a)
     a = ' '.join(a)
     return a
-#############################
-### get subcores          ###
-#############################
+    
+## Get substrate cores
+#
+#  This form of the function is used extensively in the GUI so it got it's own call. This is basically the same as getsubstrates() but returns the full dictionary
+#  @return Substrates dictionary
 def getsubcores():
-    ## this form of the functionn
-    ## is used extensively in the GUI
-    ## so it got it's own call. This
-    ## is basically the same as getsubstrates() but
-    ## returns the full dictionary
     globs = globalvars()
     if globs.custom_path: # test if a custom path is used:
          subcores = str(globs.custom_path).rstrip('/') + "/Substrates/substrates.dict"
@@ -247,9 +230,10 @@ def getsubcores():
         subcores = resource_filename(Requirement.parse("molSimplify"),"molSimplify/Substrates/substrates.dict")
     subcores = readdict(subcores)
     return subcores
-#######################
-### load bonds data ###
-#######################
+    
+## Load M-L bond length dictionary from data
+#  @param path to data file
+#  @return M-L bond length dictionary
 def loaddata(path):
     globs = globalvars()
         # loads ML data from ML.dat file and
@@ -270,9 +254,9 @@ def loaddata(path):
     f.close()
     return d
 
-###########################
-###    load backbone    ###
-###########################
+## Load backbone coordinates
+#  @param coord Name of coordination geometry
+#  @return List of backbone coordinates
 def loadcoord(coord):
     globs = globalvars()
 #    f = open(installdir+'Data/'+coord+'.dat')
@@ -290,12 +274,13 @@ def loadcoord(coord):
         b.append([float(l[0]),float(l[1]),float(l[2])])
     return b
     
-    
-###########################
-###    load core and    ###
-### convert to molecule ###
-###########################
-def core_load(usercore,mcores):
+## Load core and convert to mol3D
+#  @param usercore Name of core
+#  @param mcores Cores dictionary (reloads if not specified - default, useful when using an externally modified dictionary)
+#  @return mol3D of core, error messages
+def core_load(usercore,mcores=None):
+    if mcores == None:
+        mcores = getmcores()
     globs = globalvars()
     if '~' in usercore:
         homedir = os.path.expanduser("~")
@@ -350,7 +335,7 @@ def core_load(usercore,mcores):
         usercore = checkTMsmiles(usercore)
         # try and catch error if conversion doesn't work
         try:
-            core.OBMol = core.getOBMol(usercore,'smi') # convert from smiles
+            core.OBMol = core.getOBMol(usercore,'smistring',True) # convert from smiles
             print('Core successfully interpreted as smiles')
         except IOError:
             emsg = "We tried converting the string '%s' to a molecule but it wasn't a valid SMILES string.\n" % usercore
@@ -363,11 +348,13 @@ def core_load(usercore,mcores):
         core.ident = 'core'
     return core,emsg
 
-#############################
-###    load substrate and ###
-### convert to molecule   ### 
-#############################
-def substr_load(usersubstrate,subcores):
+## Load substrate and convert to mol3D
+#  @param usersubstrate Name of substrate
+#  @param subcores Substrates dictionary (reloads if not specified - default, useful when using an externally modified dictionary)
+#  @return mol3D of substrate, error messages
+def substr_load(usersubstrate,subcores=None):
+    if subcores == None:
+        subcores = getsubcores()
     globs = globalvars()
     if '~' in usersubstrate:
         homedir = os.path.expanduser("~")
@@ -421,7 +408,7 @@ def substr_load(usersubstrate,subcores):
         usersubstrate = checkTMsmiles(usersubstrate)
         # try and catch error if conversion doesn't work
         try:
-            substrate.OBMol = substrate.getOBMol(usersubstrate,'smi') # convert from smiles
+            substrate.OBMol = substrate.getOBMol(usersubstrate,'smistring',True) # convert from smiles
             print('Substrate successfully interpreted as smiles')
         except IOError:
             emsg = "We tried converting the string '%s' to a molecule but it wasn't a valid SMILES string.\n" % usercore
@@ -434,11 +421,14 @@ def substr_load(usersubstrate,subcores):
         substrate.ident = 'substrate'
     return substrate,emsg
    
-###########################
-###   load ligand and   ###
-### convert to molecule ###
-###########################
-def lig_load(userligand,licores):
+## Load ligand and convert to mol3D
+#  @param userligand Name of ligand
+#  @param licores Ligands dictionary (reloads if not specified - default, useful when using an externally modified dictionary)
+#  @return mol3D of ligand, error messages
+def lig_load(userligand,licores=None):
+    if licores == None:
+        licores = getlicores()
+        licores.pop("x", None)
     globs = globalvars()
     ### get groups ###
     groups = []
@@ -500,20 +490,20 @@ def lig_load(userligand,licores):
             lig.ffopt = dbentry[4][0]
     ### load from file
     elif ('.mol' in userligand or '.xyz' in userligand or '.smi' in userligand or '.sdf' in userligand):
-        flig = resource_filename(Requirement.parse("molSimplify"),"molSimplify/" +userligand)
-        if glob.glob(flig):
-            ftype = flig.split('.')[-1]
+        #flig = resource_filename(Requirement.parse("molSimplify"),"molSimplify/" +userligand)
+        if glob.glob(userligand):
+            ftype = userligand.split('.')[-1]
             # try and catch error if conversion doesn't work
             try:
                 print('ligand is an '+ftype+' file')
-                lig.OBMol = lig.getOBMol(flig,ftype+'f') # convert from file
+                lig.OBMol = lig.getOBMol(userligand,ftype+'f') # convert from file
                 # generate coordinates if not existing
                 lig.charge = lig.OBMol.GetTotalCharge()
                 print('Ligand successfully converted to OBMol')
             except IOError:
                 emsg = 'Failed converting file ' +userligand+' to molecule..Check your file.\n'
                 return False,emsg
-            lig.ident = flig.rsplit('/')[-1]
+            lig.ident = userligand.rsplit('/')[-1]
             lig.ident = lig.ident.split('.'+ftype)[0]
         else:
             emsg = 'Ligand file '+userligand+' does not exist. Exiting..\n'
@@ -521,11 +511,13 @@ def lig_load(userligand,licores):
             return False,emsg
     ### if not, try interpreting as SMILES string
     else:
-        try:
-            lig.OBMol = lig.getOBMol(userligand,'smistring') # convert from smiles
+        try: 
+            lig.getOBMol(userligand,'smistring',True) # convert from smiles
+            lig.convert2mol3D()
+            assert lig.natoms            
             lig.charge = lig.OBMol.GetTotalCharge()
             print('Ligand successfully interpreted as SMILES')
-        except IOError:
+        except:
             emsg = "We tried converting the string '%s' to a molecule but it wasn't a valid SMILES string.\n" % userligand
             emsg += "Furthermore, we couldn't find the ligand structure: '%s' in the ligands dictionary. Try again!\n" % userligand
             emsg += "\nAvailable ligands are: %s\n" % getligs()
@@ -536,10 +528,10 @@ def lig_load(userligand,licores):
     lig.name = userligand
     return lig,emsg
 
-####################################
-###   load binding species and   ###
-#####   convert to molecule    #####
-####################################
+## Load binding species and convert to mol3D
+#  @param userbind Name of binding species
+#  @param bindcores Binding species dictionary (reloads if not specified - default, useful when using an externally modified dictionary)
+#  @return mol3D of binding species, error messages
 def bind_load(userbind,bindcores):
     globs = globalvars()
     if '~' in userbind:
@@ -601,8 +593,9 @@ def bind_load(userbind,bindcores):
             return False,False,emsg
     return bind, bsmi, emsg
 
-
-# write input file
+## Write input file from arguments
+#  @param args Namespace of arguments
+#  @param fname File name
 def getinputargs(args,fname):
     # list with arguments
     # write input args
@@ -629,60 +622,67 @@ def getinputargs(args,fname):
                     f.write(str(getattr(args, arg)))
                 f.write('\n')
     f.close()
-#####################################
-###    load plugin definitions    ###
-#####################################
+
+## Load plugin definitions
 def plugin_defs():
     globs = globalvars()
     plugin_path = resource_filename(Requirement.parse("molSimplify"),"molSimplify/plugindefines_reference.txt")
     return plugin_path
 
-#####################################
-###   file/folder name control   ###
-####################################
-def get_name(args,rootdir,core,ligname,bind = False,bsmi = False):
-    ### DEPRECIATED, USE NAME_COMPLEX instead
-    # reads in argument namespace
-    # and chooses an appropriate name
-    # bind_ident is used to pass binding
-    # species information 
-    print('the root directory for this calc is '+ (rootdir))
-    # check if smiles string in binding species
-    if args.bind:
-        if bsmi:
-            if args.nambsmi: # if name specified use it in file
-                fname = rootdir+'/'+core.ident[0:3]+ligname+args.nambsmi[0:2]
-                if args.name:
-                    fname = rootdir+'/'+args.name+args.nambsmi[0:2]
-            else: # else use default
-                fname = rootdir+'/'+core.ident[0:3]+ligname+'bsm' 
-                if args.name:
-                   fname = rootdir+'/'+args.name+'bsm'
-        else: # else use name from binding in dictionary
-            fname = rootdir+'/'+core.ident[0:3]+ligname+bind.ident[0:2]
-            if args.name:
-                fname = rootdir+'/'+args.name + bind.ident[0:2]
-    else:
-        if globs.debug:
-            print('the root calculation directory is' + str(rootdir))
-        fname = rootdir+'/'+core.ident[0:3]+ligname
-        if args.name:
-            fname = rootdir+'/'+args.name
+#def get_name(args,rootdir,core,ligname,bind = False,bsmi = False):
+    #### DEPRECIATED, USE NAME_COMPLEX instead
+    ## reads in argument namespace
+    ## and chooses an appropriate name
+    ## bind_ident is used to pass binding
+    ## species information 
+    #print('the root directory for this calc is '+ (rootdir))
+    ## check if smiles string in binding species
+    #if args.bind:
+        #if bsmi:
+            #if args.nambsmi: # if name specified use it in file
+                #fname = rootdir+'/'+core.ident[0:3]+ligname+args.nambsmi[0:2]
+                #if args.name:
+                    #fname = rootdir+'/'+args.name+args.nambsmi[0:2]
+            #else: # else use default
+                #fname = rootdir+'/'+core.ident[0:3]+ligname+'bsm' 
+                #if args.name:
+                   #fname = rootdir+'/'+args.name+'bsm'
+        #else: # else use name from binding in dictionary
+            #fname = rootdir+'/'+core.ident[0:3]+ligname+bind.ident[0:2]
+            #if args.name:
+                #fname = rootdir+'/'+args.name + bind.ident[0:2]
+    #else:
+        #if globs.debug:
+            #print('the root calculation directory is' + str(rootdir))
+        #fname = rootdir+'/'+core.ident[0:3]+ligname
+        #if args.name:
+            #fname = rootdir+'/'+args.name
 
-    return fname
+    #return fname
 
-
-
-def name_complex(rootdir,core,ligs,ligoc,sernum,args,bind= False,bsmi=False):
+## Generate complex name (this is actually used instead of namegen.py)
+#  @param rootdir Root directory
+#  @param core mol3D of core
+#  @param ligs List of ligand names
+#  @param ligoc List of ligand occurrences
+#  @param sernum Complex serial number
+#  @param args Namespace of arguments
+#  @param bind Flag for binding species (default False)
+#  @param bsmi Flag for SMILES binding species (default False)
+#  @return Complex name
+def name_complex(rootdir,core,ligs,ligoc,sernum,args,nconf=False,sanity=False,bind= False,bsmi=False):
     ## new version of the above, designed to 
     ## produce more human and machine-readable formats
     if args.name: # if set externerally
         name = rootdir+'/'+args.name
     else:
+        center = ''
+        if sanity:
+            center += 'badjob_'
         try:
-            center = core.getAtom(0).symbol().lower()
+            center += core.getAtom(0).symbol().lower()
         except:
-            center = str(core).lower()
+            center += str(core).lower()
         name = rootdir + '/' + center
         if args.oxstate:
             if args.oxstate in romans.keys():
@@ -706,12 +706,22 @@ def name_complex(rootdir,core,ligs,ligoc,sernum,args,bind= False,bsmi=False):
             else:
                 name += '_' + str(lig) + '_' + str(ligoc[i])
         name += "_s_"+str(spin)
+        if nconf and args.nconfs > 1:
+            name += "_conf_"+str(nconf)
         if args.bind:
             if bsmi:
                 if args.nambsmi: # if name specified use it in file
                     name += "_" + +args.nambsmi[0:2]
     return name
 
+## Generate transition state name
+#  @param rootdir Root directory
+#  @param core mol3D of core
+#  @param subst mol3D of substrate
+#  @param args Namespace of arguments
+#  @param bind Flag for binding species (default False)
+#  @param bsmi Flag for SMILES binding species (default False)
+#  @return Transition state name
 def name_TS(rootdir,core,subst,args,bind= False,bsmi=False):
     ## new version of the above, designed to 
     ## produce more human and machine-readable formats
@@ -744,10 +754,7 @@ def name_TS(rootdir,core,subst,args,bind= False,bsmi=False):
                     name += "_" + +args.nambsmi[0:2]
     return name
 
-##############################################
-### function to copy ligands, bind and     ###
-### cores to user-specified path       ###
-##############################################
+## Copies ligands, binding species and cores to user-specified path
 def copy_to_custom_path():
     globs = globalvars()
     if not globs.custom_path:
