@@ -1,13 +1,10 @@
-# Written by JP Janet for HJK Group
-# Dpt of Chemical Engineering, MIT
+## @file nn_prep.py
+#  Helper routines for ANN integration
+#  
+#   Written by JP Janet for HJK Group
+#  
+#  Dpt of Chemical Engineering, MIT
 
-##########################################################
-####### This script is a collection of helper ############
-########  routines for ANN integration in    #############
-########           molsimplify               #############
-##########################################################
-
-# import custom modules
 from molSimplify.Scripts.geometry import *
 from molSimplify.Scripts.io import *
 from molSimplify.Informatics.decoration_manager import*
@@ -15,10 +12,8 @@ from molSimplify.Classes.globalvars import *
 from molSimplify.Informatics.graph_analyze import *
 from molSimplify.python_nn.ANN import *
 import numpy
-# import standard modules
 import openbabel
 
-
 def get_bond_order(OBMol,connection_atoms,mol):
     ## informs the ANN
     ## of the highest bond
@@ -310,12 +305,15 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
         ax_ki = get_truncated_kier(ax_lig3D,ax_lig3D.cat)
         eq_EN = get_lig_EN(eq_lig3D,eq_lig3D.cat)
         ax_EN = get_lig_EN(ax_lig3D,ax_lig3D.cat)
-        eq_bo = get_bond_order(eq_lig3D.OBmol.OBMol,eq_lig3D.cat,eq_lig3D)
-        ax_bo = get_bond_order(ax_lig3D.OBmol.OBMol,ax_lig3D.cat,ax_lig3D)
+        eq_bo = get_bond_order(eq_lig3D.OBMol,eq_lig3D.cat,eq_lig3D)
+        ax_bo = get_bond_order(ax_lig3D.OBMol,ax_lig3D.cat,ax_lig3D)
 
-
-        eq_charge = eq_lig3D.OBmol.charge
-        ax_charge = ax_lig3D.OBmol.charge
+        if axial_ligs[0] in ['carbonyl','cn']:
+            ax_bo = 3
+        if equitorial_ligs[0] in ['carbonyl','cn']:
+            eq_bo = 3
+        eq_charge = eq_lig3D.OBMol.GetTotalCharge()
+        ax_charge = ax_lig3D.OBMol.GetTotalCharge()
 
          
         ## preprocess:
@@ -354,7 +352,7 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
 
 
         nn_excitation = [0,0,0,0,0, # metals co/cr/fe/mn/ni                 #1-5
-                   ox,alpha,eq_charge,ax_charge, #ox/alpha/eqlig charge/axlig charge #6-9
+                   ox,alpha,ax_charge,eq_charge, #ox/alpha/axlig charge/eqlig charge #6-9
                    ax_dent,eq_dent,# ax_dent/eq_dent/ #10-11
                    0,0,0,0, # axlig_connect: Cl,N,O,S #12 -15
                    0,0,0,0, # eqliq_connect: Cl,N,O,S #16-19
@@ -362,7 +360,7 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
                    ax_bo,eq_bo, #axlig_bo, eqliq_bo #22-23
                    ax_ki,eq_ki]#axlig_ki, eqliq_kii #24-25
 	slope_excitation = [0,0,0,0,0, # metals co/cr/fe/mn/ni                 #1-5
-                   ox,eq_charge,ax_charge, #ox/eqlig charge/axlig charge #6-8
+                   ox,ax_charge,eq_charge, #ox/axlig charge/eqlig charge #6-8
                    ax_dent,eq_dent,# ax_dent/eq_dent/ #9-10
                    0,0,0,0, # axlig_connect: Cl,N,O,S #11 -14
                    0,0,0,0, # eqliq_connect: Cl,N,O,S #15-18
@@ -409,8 +407,8 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
 
         ANN_attributes.update({'ANN_dist_to_train':train_dist} )
         ANN_attributes.update({'ANN_closest_train':best_row} )
-        print('distance to training data is ' + "{0:.2f}".format(train_dist) + ' ANN trust: ' +str(ANN_trust))
-        print(' with closest training row ' + best_row[:-2])
+        print('distance to training data is ' + "{0:.2f}".format(train_dist) + ' ANN trust: ' +"{0:.2f}".format(ANN_trust))
+        print(' with closest training row ' + best_row[:-2] + ' at HFX ' + str(best_row[-2:]) + '%')
         ANN_trust = 'not set'
         if float(train_dist)< 0.25:
             print('ANN results should be trustworthy for this complex ')
@@ -428,7 +426,7 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
         ## engage ANN
         delta = 0  
 
-        delta = get_splitting(nn_excitation)
+        delta,scaled_excitation = get_splitting(nn_excitation)
         ## report to stdout
         if delta[0] < 0 and not high_spin:
             if abs(delta[0]) > 5:
@@ -442,6 +440,7 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
                     print('warning, ANN predicts a near degenerate ground state for this complex')
         print("ANN predicts a spin splitting (HS - LS) of " + "{0:.2f}".format(float(delta[0])) + ' kcal/mol')
         ANN_attributes.update({'pred_split_ HS_LS':delta[0]})
+        ANN_attributes.update({'normed_excitation':scaled_excitation})
         ## reparse to save attributes
         ANN_attributes.update({'This spin':spin})
         if delta[0] < 0 and (abs(delta[0]) > 5):
@@ -451,15 +450,19 @@ def ANN_preproc(args,ligs,occs,dents,batslist,tcats,licores):
         else:
                 ANN_attributes.update({'ANN_gound_state':'dgen ' + str(spin_ops)})
 
-        r = 0
+        
+        r_ls = get_ls_dist(nn_excitation)
+        r_hs = get_hs_dist(nn_excitation)
         if not high_spin:
-            r = get_ls_dist(nn_excitation)
+            r = r_ls
         else:
-            r = get_hs_dist(nn_excitation)
+            r = r_hs
 
         print('ANN bond length is predicted to be: '+"{0:.2f}".format(float(r)) + ' angstrom')
         ANN_attributes.update({'ANN_bondl':r[0]})
-
+        print('ANN low spin bond length is predicted to be: '+"{0:.2f}".format(float(r_ls)) + ' angstrom')
+        print('ANN high spin bond length is predicted to be: '+"{0:.2f}".format(float(r_hs)) + ' angstrom')
+        
 	### use ANN to predict fucntional sensitivty
         HFX_slope = 0 
         HFX_slope = get_slope(slope_excitation)
@@ -533,8 +536,9 @@ def spin_classify(metal,spin,ox):
     return high_spin,spin_ops
 
 def get_splitting(excitation):
-    delta = simple_splitting_ann(excitation)
-    return delta
+    #print(excitation)
+    delta,scaled_excitation = simple_splitting_ann(excitation)
+    return delta,scaled_excitation
 def get_slope(slope_excitation):
     HFX = simple_slope_ann(slope_excitation)    
     return HFX
