@@ -1,11 +1,9 @@
-# Written by Tim Ioannidis for HJK Group
-# Dpt of Chemical Engineering, MIT
-
-##########################################################
-##########  Top level script that coordinates  ###########
-##########    generation of structues, input   ###########
-##########         files, jobscripts           ###########
-##########################################################
+## @file rungen.py
+#  Top level script that coordinates generation of all files
+#  
+#  Written by Tim Ioannidis for HJK Group
+#
+#  Dpt of Chemical Engineering, MIT
 
 from structgen import *
 from molSimplify.Scripts.io import *
@@ -81,13 +79,7 @@ def getconstsample(no_rgen,args,licores,coord):
 def constrgen(rundir,args,globs):
     emsg = False
     # load global variables
-    mcores = getmcores()
     licores = getlicores()
-    bindcores = getbcores()
-
-
-    # remove empty ligand
-    licores.pop("x", None)
     print 'Random generation started..\n\n'
     # if ligand constraint apply it now
     ligs0 = []
@@ -100,7 +92,7 @@ def constrgen(rundir,args,globs):
     if args.lig:
         for i,l in enumerate(args.lig):
             ligs0.append(l)
-            ligentry,emsg = lig_load(l,licores) # check ligand
+            ligentry,emsg = lig_load(l) # check ligand
             # update ligand
             if ligentry:
                 args.lig[i] = ligentry.name
@@ -182,9 +174,11 @@ def constrgen(rundir,args,globs):
         emsg = rungen(rundir,args,False,globs) # run structure generation
     return args, emsg
 
-################################################################
-### generates multiple runs for different ox and spin states ###
-################################################################
+## Generates multiple runs for different oxidation and spin states
+#  @param rundir Run directory
+#  @param args Namespace of arguments
+#  @param globs Global variables
+#  @return Error messages
 def multigenruns(rundir,args,globs):
     emsg = False
     args.jid = 0 # initilize global name identifier
@@ -253,27 +247,29 @@ def multigenruns(rundir,args,globs):
             emsg = rungen(rundir,args,fname,globs)
     return emsg
 
-#########################################################
-### checks for multiple ligands specified in one file ###
-#########################################################
+## Check for multiple ligands specified in one file
+#  @param ligs List of ligands
+#  @return Ligand list, connecting atoms, multiple ligand flag
 def checkmultilig(ligs):
     mligs = []
     tcats = []
     multidx = -1
     # loop over ligands
+    print(ligs)
     for i,lig in enumerate(ligs):
         connatoms = []
         if '.smi' in lig:
             if '~' in lig:
                 lig = lig.replace('~',os.path.expanduser("~"))
             # read molecule
-            flig = resource_filename(Requirement.parse("molSimplify"),"molSimplify/" +lig)
-            if glob.glob(flig):
-                f = open(flig,'r')
+            if glob.glob(lig):
+                print('found ligand file')
+                f = open(lig,'r')
                 s = f.read().splitlines()
                 for ss in s:
                     ss = ss.replace('\t',' ')
                     sf = filter(None,ss.split(' '))
+                    print(sf)
                     if len(sf) > 0:
                         connatoms.append(sf[-1])
                         multidx = i
@@ -289,7 +285,6 @@ def checkmultilig(ligs):
         else:
             mligs.append([lig])
         tcats.append(connatoms)
-    print mligs
     ligandslist = list(itertools.product(*mligs))
     # convert tuple to list
     llist = []
@@ -299,19 +294,19 @@ def checkmultilig(ligs):
             for l1 in l0:
                 loclist.append(l1)
             llist.append(loclist)
-
+    print(llist)
+    print(tcats)
     return llist,tcats,multidx
 
-#################
-### draw mode ###
-#################
+## Draw mode supervisor
+#  @param args Namespace of arguments
+#  @param rundir Run directory
 def draw_supervisor(args,rundir):
     if args.lig:
         print('Due to technical limitations, we will draw only the first ligand.') 
         print('To view multiple ligands at once, consider using the GUI instead.')
-        licores = getlicores()
         l = args.lig[0]
-        lig, emsg = lig_load(l,licores)
+        lig, emsg = lig_load(l)
         lig.draw_svg(l)
     elif args.core:
         if len(args.core) > 1:
@@ -319,23 +314,24 @@ def draw_supervisor(args,rundir):
         print('Drawing the core.')
         if args.substrate:
             print('Due to technical limitations, we can draw only one structure per run. To draw the substrate, run the program again.')
-        mcores = getmcores()
-        cc, emsg = core_load(args.core[0],mcores)
+        cc, emsg = core_load(args.core[0])
         cc.draw_svg(args.core[0])
     elif args.substrate:
         if len(args.substrate) > 1:
             print('Due to technical limitations, we will draw only the first substrate.')
         print('Drawing the substrate.')
         print args.substrate[0]
-        subcores = getsubcores()
-        substrate, emsg = substr_load(args.substrate[0],subcores)
+        substrate, emsg = substr_load(args.substrate[0])
         substrate.draw_svg(args.substrate[0])
     else:
 		print('You have not specified anything to draw. Currently supported: ligand, core, substrate')
 
-##############################################
-### normal structure generation of complex ###
-##############################################
+## Normal structure generation
+#  @param rundir Run directory
+#  @param args Namespace of arguments
+#  @param chspfname Folder name for charges and spins
+#  @param globs Global variables
+#  @return Error messages
 def rungen(rundir,args,chspfname,globs):
     try:
         from Classes.mWidgets import qBoxFolder
@@ -344,13 +340,6 @@ def rungen(rundir,args,chspfname,globs):
     except ImportError:
         args.gui = False
     emsg = False
-    mcores = getmcores()
-    licores = getlicores()
-    bindcores = getbcores()
-    cc, emsg = core_load(args.core,mcores)
-    if emsg:
-        return emsg
-    mname = cc.ident
     globs.nosmiles = 0 # reset smiles ligands for each run
     # check for specified ligands/functionalization
     ligocc = []
@@ -391,7 +380,7 @@ def rungen(rundir,args,chspfname,globs):
                 ligocc.append('1')
             lig = ''
             for i,l in enumerate(ligands):
-                ligentry,emsg = lig_load(l,licores)
+                ligentry,emsg = lig_load(l)
                 # update ligand
                 if ligentry:
                     ligands[i] = ligentry.name
@@ -411,20 +400,9 @@ def rungen(rundir,args,chspfname,globs):
             lig = ''
             ligocc = ''
     ##### fetch smart name
-        fname = name_complex(rundir,args.core,ligands,ligocc,mcount,args,bind=args.bind,bsmi=args.nambsmi)
+        fname = name_complex(rundir,args.core,ligands,ligocc,mcount,args,nconf=False,sanity=False,bind=args.bind,bsmi=args.nambsmi)
         if globs.debug:
             print('fname is ' + str(fname))
-
-        #        if args.bind:
-                # create folder for runs and check if it already exists
-        #            if args.nambsmi:
-        #                rootdir = rundir+mname[0:4]+lig+args.nambsmi[0:3]
-        #            elif not args.bind in bindcores.keys():
-        #                rootdir = rundir+mname[0:4]+lig+'bsmi'
-        #            else:
-        #                rootdir = rundir+mname[0:4]+lig+args.bind[0:4]
-        #        else:
-        #            rootdir = rundir+mname[0:4]+lig
         rootdir = fname
         # check for charges/spin
         rootcheck = False
@@ -578,9 +556,7 @@ def rungen(rundir,args,chspfname,globs):
                 elif args.jsched in 'SGE Sungrid sge':
                     sgejobgen(args,jobdirs)
                     print 'SGE jobscripts generated!'
-            if this_diag.sanity: # move to separate subdirectory if generated structure was bad
-                fname = rootdir.rsplit('/',1)[-1]
-                shutil.move(rootdir,rundir+'/badjobs/'+fname)
+
             elif multidx != -1: # if ligand input was a list of smiles strings, write good smiles strings to separate list
                 try:
                     f = open(ligfilename+'-good.smi','a')
@@ -596,15 +572,16 @@ def rungen(rundir,args,chspfname,globs):
                 print 'Folder '+rootdir+' was skipped..\n'
     return emsg    
 
-##############################################
-### transition state generation ###
-##############################################
-
+## Transition state generation
+#  @param rundir Run directory
+#  @param args Namespace of arguments
+#  @param chspfname Folder name for charges and spins
+#  @param globs Global variables
+#  @return Error messages
 def tsgen_supervisor(rundir,args,chspfname,globs):
 	emsg = False
 	# load specified core into a mol3D object
-	mcores = getmcores()
-	cc, emsg = core_load(args.core,mcores)
+	cc, emsg = core_load(args.core)
 	if emsg:
 		return emsg	
 	cc.convert2mol3D()
